//LCD handler  (c) Zoltan Benesocky

#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include <string.h>
#include "BooleConst.h"
#include "SPI_hlr.h"
#include "Timer_std_hlr.h"
#include "LCD_hlr.h"

//PORTC
#define SS_LCD 3
#define SET_SS_LCD PORTC |= (1 << SS_LCD)
#define CLR_SS_LCD PORTC &= ~(1 << SS_LCD)

volatile int LCD_WR_RDY;
volatile int StrIndex;
volatile char * StrPtr;
volatile char ITstr [20];
volatile unsigned char LCDline;
volatile unsigned char LCDpos;

static char tmpstr1[20];
static char tmpstr2[20];

void LCD_DATA(char data);
void SetLineAndPos(unsigned char line, unsigned char pos);

// Must be include to void Timer_std_ItExtension(void) in Timer_std_ext.c
// This will be called by ISR(TIMER0_COMPA_vect)
// May it call periodically from other timer IT (period: ~0.5msec)
void LCD_wr_from_IT(void)
{
	char StrChr;
	if(~SPI_Busy)	// No use SPI till busy
	{
		if(!LCD_WR_RDY)							//Handling of LCD
		{
			if(StrIndex == -1)
			{
				SPI_Busy = TRUE;
				SetLineAndPos(LCDline, LCDpos);				//first set line
				SPI_Busy = FALSE;
			}
			else
			{					
				StrChr = ITstr[StrIndex];		//get next character from string
				if(StrChr != 0)
				{
					SPI_Busy = TRUE;
					LCD_DATA(StrChr);			//write character to LCD
					SPI_Busy = FALSE;
				}
				else
				{
					LCD_WR_RDY = 1;				// sign to finished writing to LCD
				}
			}
			StrIndex++;
		}
	}
}

// t[msec]
void Delaysw(long t)
{
	t = (long)(160*(float)t);
	for(long i = 0; i<t; i++)
	{
		char tmp =	PORTD;
	}	 
}


//D7 D6 D5 D4 RS E X X
#define RS 0x08
#define E 0x04

#define CLR_DISP 0x01
#define CR 0x02
#define CGRAM_ADDR 0x40
#define DDRAM_ADDR 0x80
#define LINE1 DDRAM_ADDR
#define LINE2 (DDRAM_ADDR | 0x40)

//write to shiftregister + register connected to LCD interface
void LCD_REG_WR(unsigned char data)
{
	CLR_SS_LCD;
	SPI_MasterTransmit(data);	//write to shiftregister
	SET_SS_LCD;					//write content of shr to reg
}

// write one half of LCD command
void LCD_COMM_HALF(unsigned char com4bits)
{
	unsigned char com;
	com = (com4bits << 4);
	LCD_REG_WR(com);
	com = (com4bits << 4) | E;
	LCD_REG_WR(com);
	com = (com4bits << 4);
	LCD_REG_WR(com);
}

// write LCD command
void LCD_COMM(unsigned char com)
{
	unsigned char com_tmp;
	com_tmp = com >> 4;
	LCD_COMM_HALF(com_tmp);
	com_tmp = com & 0x0f;
	LCD_COMM_HALF(com_tmp);
}

// write one half of LCD data
void LCD_DATA_HALF(char data4bits)
{
	unsigned char data;
	data = (data4bits << 4) | RS;
	LCD_REG_WR(data);
	data = (data4bits << 4) | RS | E;
	LCD_REG_WR(data);
	data = (data4bits << 4) | RS;
	LCD_REG_WR(data);
}

// write LCD data
void LCD_DATA(char data)
{
	char data_tmp;
	data_tmp = data >> 4;		// MSB first
	LCD_DATA_HALF(data_tmp);
	data_tmp = data & 0x0f;
	LCD_DATA_HALF(data_tmp);
}


void LCD_init(void)
{
	DDRC |= (1 << SS_LCD);		//SS_LCD direction: out
	SET_SS_LCD;					//SS_LCD inactive
	SPI_MasterInit();
	LCD_WR_RDY = 1;	
	// set 4 bits mode
	Delaysw(150);
	LCD_COMM_HALF(0x03);
	Delaysw(50);
	LCD_COMM_HALF(0x03);
	Delaysw(50);
	LCD_COMM_HALF(0x03);
	Delaysw(50);
	LCD_COMM_HALF(0x02);
	Delaysw(50);
	// 4 bits mode ready
	LCD_COMM(0x28);		//function set 0 0 1 DL N F x x   2 lines
	Delaysw(50);
	LCD_COMM(0x08);		//display off  0 0 0 0 1 D C B   disp. off, curs. off, blink off
	Delaysw(50);
	LCD_COMM(0x01);		//clear and home 0 0 0 0 0 0 0 1
	Delaysw(50);
	//LCD_COMM(0x0c);	//display on, no cursor, no blink
	//DelayTim0(t_5ms);
	LCD_COMM(0x0c);		// dp. on, curs. on, blink on
	Delaysw(50);
	LCD_COMM(0x06);		// Entry mode 0 0 0 0 0 1 I/D S dp. on, curs. on, blink on
	Delaysw(50);
}

//Set LCD line (1 or 2) and pos (0-15) to write

void SetLineAndPos(unsigned char line, unsigned char pos)
{
	if(line == 1)
	{
		LCD_COMM(LINE1+pos);
	}
	else
	{
		LCD_COMM(LINE2+pos);
	}
}


void CLR_DISPLAY(void)
{
	LCD_COMM(CLR_DISP);
}

//copy l pieces of characters
void cpy_str(char * sd, char * ss, int l)  // sd[l-1:0] = ss[l-1:0]
{	
	int i;
	int len;
	int sl;
	
	sl = strlen(ss);
	
	if(l > sl)
	{
		len = sl;
	}
	else
	{
		len = l;
	}
	
	for(i=0; i<len; i++)
	{
		sd[i] = ss[i];
	}
	sd[i]=0;
}

//complete string shorter then 16 char with space
void complete_str_n(char * s, int n)	
{
	int l = strlen(s);
	if(l < n)
	{
		for(int i=0; i<=(n-l); i++ )
		{
			strcat(s," ");
		}
	}	
}

//write string to LCD line
void LCD_write_str(char * s, unsigned char line)
{
	//char tmpstr[20];
	
	cpy_str(tmpstr1,s,16);
	complete_str_n(tmpstr1,16);
	
	do{} while(!LCD_WR_RDY);
		
	strcpy(ITstr,tmpstr1);
	//StrPtr = ITstr;
	StrIndex = -1;
	LCDline = line;
	LCDpos = 0;
	LCD_WR_RDY = 0;
}

void LCD_write_str_pos(char * s, unsigned char line, unsigned char pos)
{
	//char tmpstr[20];
	
	cpy_str(tmpstr1,s,16);
	//complete_str_n(tmpstr1,16);;
	
	do{} while(!LCD_WR_RDY);
	
	strcpy(ITstr,tmpstr1);
	//StrPtr = ITstr;
	StrIndex = -1;
	LCDline = line;
	LCDpos = pos;
	LCD_WR_RDY = 0;
}

void LCD_write_int_pos(int int_var, unsigned char line, unsigned char pos)
{
	itoa(int_var,tmpstr1,10);	
	complete_str_n(tmpstr1,5);
	
	do{} while(!LCD_WR_RDY);
	
	strcpy(ITstr,tmpstr1);

	StrIndex = -1;
	LCDline = line;
	LCDpos = pos;
	LCD_WR_RDY = 0;
}

void LCD_write_double_pos(double double_var, char with, unsigned char prec, unsigned char line, unsigned char pos)
{
	dtostrf(double_var,with,prec,tmpstr1);
	complete_str_n(tmpstr1,with);
	do{} while(!LCD_WR_RDY);
	
	strcpy(ITstr,tmpstr1);

	StrIndex = -1;
	LCDline = line;
	LCDpos = pos;
	LCD_WR_RDY = 0;
}

//write string and integer to LCD line
void LCD_write_str_int(char * s, int data, unsigned char side, unsigned char line)
{
	cli();	
	cpy_str(tmpstr1,s,14);
	itoa(data,tmpstr2,10);
	if(side == right)
	{
		strcat(tmpstr1,tmpstr2);	
	}
	else
	{
		strcat(tmpstr2,tmpstr1);
		strcpy(tmpstr1,tmpstr2);
	}
	
	complete_str_n(tmpstr1,16);;
	sei();
	do{} while(!LCD_WR_RDY);
		
	strcpy(ITstr,tmpstr1);

	StrIndex = -1;
	LCDline = line;
	LCDpos = 0;
	LCD_WR_RDY = 0;
}

//write string to LCD line1 and line2 
void LCD_write_longstr(char * s)
{	
	//char tmpstr[16];
	char *strptr = s+16;
	int len = strlen(s);
	LCD_write_str(s,line1);
	if(len>16)
	{
		strncpy(tmpstr1,strptr,len-16);
		tmpstr1[len-16] = 0;
		complete_str_n(tmpstr1,16);
		LCD_write_str(tmpstr1,line2);
	}
}

//set back light (value: 0-15)
void LCD_light(unsigned char bright)
{
	SetPWM0(bright);
}