//LCD handler  (c) Zoltan Benesocky


#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include <string.h>
#include "SPI_tools.h"
#include "BooleConst.h"
#include "LCD_hlr.h"

//PORTD
#define PWM 5

//PORTC
#define SS_LCD 3
#define SET_SS_LCD PORTC |= (1 << SS_LCD)
#define CLR_SS_LCD PORTC &= ~(1 << SS_LCD)

//use TIMER0
//        D7     D6     D5     D4     D3    D2   D1    D0
//TCCR0A: COM0A1 COM0A0 COM0B1 COM0B0 -     -    WGM01 WGM00
//        0      0      1      0      0     0    1     1
//TCCR0B: FOC0A  FOC0B  -      -      WGM02 CS02 CS01  CS00
//        0      0      0      0      1     1    0     0
//Fast PWM mode, div=256, ORC0A = 15 (31300 Hz, 16/31300 = 0.512 msec, 1.95kHz)

#define tim0_modeA 0x23	//PWM on OC0B
#define tim0_modeB 0x0c
#define t_32ms 62
#define t_15ms 29
#define t_5ms 9
#define t_05ms 0

static volatile int tim0;
static volatile char ovr0;
static volatile char tim_chr;
static volatile int LCD_WR_RDY;
static volatile int StrIndex;
static volatile char * StrPtr;
static volatile char ITstr [20];
static volatile int LCDline;
static char tmpstr1[20];
static char tmpstr2[20];

void LCD_DATA(char data);
void SetLine(char line);


void TIMER0_Init(void)
{
	TCCR0A = tim0_modeA;
	TCCR0B = tim0_modeB;
	OCR0A = 0x0f;  //f = 8MHz/(256*16) = 1.95kHz (0.512ms)
	TIMSK0 = TIMSK0 | (1 << OCIE0A); // output compare IT enabled
}

void LCD_wr_from_IT(void)
{
	char StrChr;
	if(~SPI_Busy)
	{
		if(!LCD_WR_RDY)							//Handling of LCD
		{
			if(StrIndex == -1)
			{
				SPI_Busy = TRUE;
				SetLine(LCDline);				//first set line
				SPI_Busy = FALSE;
			}
			else
			{					
				StrChr = ITstr[StrIndex];		//get next character from string
				if(StrChr != 0)
				{
					SPI_Busy = TRUE;
					LCD_DATA(StrChr);			//write character to LCD
					SPI_Busy = FALSE;
				}
				else
				{
					LCD_WR_RDY = 1;				// sign to finished writing to LCD
				}
			}
			StrIndex++;
		}
	}
}


void delay_cnt_in_IT(void)
{
	if(tim0 > 0)
	{
		tim0--;
	}
	else
	{
		ovr0 = 1;
	}
}


// Timer0 compareA IT, 1.95kHz (0.512ms)
ISR(TIMER0_COMPA_vect)
{
	LCD_wr_from_IT();
	delay_cnt_in_IT();
#ifdef _Timer0ItExtension
	Timer0ItExtension();
#endif		
}


void StartTime0(int t)
{
	tim0 = 2*t;
	ovr0 = 0;
}

unsigned char Time0Done(void)
{
	unsigned char timeout;
	timeout = ovr0;
	if(ovr0)
	{
		ovr0 = 0;
	}
	return(timeout);
}


void DelayTim0(int t)
{
	tim0 = t;
	ovr0 = 0x00;
	do
	{
	}while(!ovr0);
}

// t[msec]
Delaysw(long t)
{
	t = (long)(160*(float)t);
	for(long i = 0; i<t; i++)
	{
		char tmp =	PORTD;
	}	 
}


//D7 D6 D5 D4 RS E X X
#define RS 0x08
#define E 0x04

#define CLR_DISP 0x01
#define CR 0x02
#define CGRAM_ADDR 0x40
#define DDRAM_ADDR 0x80
#define LINE1 DDRAM_ADDR
#define LINE2 DDRAM_ADDR | 0x40

//write to shiftregister + register connected to LCD interface
void LCD_REG_WR(unsigned char data)
{
	CLR_SS_LCD;
	SPI_MasterTransmit(data);	//write to shiftregister
	SET_SS_LCD;					//write content of shr to reg
}

// write one half of LCD command
void LCD_COMM_HALF(unsigned char com4bits)
{
	unsigned char com;
	com = (com4bits << 4);
	LCD_REG_WR(com);
	com = (com4bits << 4) | E;
	LCD_REG_WR(com);
	com = (com4bits << 4);
	LCD_REG_WR(com);
}

// write LCD command
void LCD_COMM(unsigned char com)
{
	unsigned char com_tmp;
	com_tmp = com >> 4;
	LCD_COMM_HALF(com_tmp);
	com_tmp = com & 0x0f;
	LCD_COMM_HALF(com_tmp);
}

// write one half of LCD data
void LCD_DATA_HALF(char data4bits)
{
	unsigned char data;
	data = (data4bits << 4) | RS;
	LCD_REG_WR(data);
	data = (data4bits << 4) | RS | E;
	LCD_REG_WR(data);
	data = (data4bits << 4) | RS;
	LCD_REG_WR(data);
}

// write LCD data
void LCD_DATA(char data)
{
	char data_tmp;
	data_tmp = data >> 4;		// MSB first
	LCD_DATA_HALF(data_tmp);
	data_tmp = data & 0x0f;
	LCD_DATA_HALF(data_tmp);
}


void LCD_init(void)
{
	DDRD |= (1 << PWM);			// PWM direction: out
	PORTD &= ~(1 << PWM);		// PWM = 0
	DDRC |= (1 << SS_LCD);		//SS_LCD direction: out
	SET_SS_LCD;					//SS_LCD inactive
	SPI_MasterInit();
	TIMER0_Init();
	LCD_WR_RDY = 1;
	
	// set 4 bits mode
	Delaysw(150);
	LCD_COMM_HALF(0x03);
	Delaysw(50);
	LCD_COMM_HALF(0x03);
	Delaysw(50);
	LCD_COMM_HALF(0x03);
	Delaysw(50);
	LCD_COMM_HALF(0x02);
	Delaysw(50);
	// 4 bits mode ready
	LCD_COMM(0x28);		//function set 0 0 1 DL N F x x   2 lines
	Delaysw(50);
	LCD_COMM(0x08);		//display off  0 0 0 0 1 D C B   disp. off, curs. off, blink off
	Delaysw(50);
	LCD_COMM(0x01);		//clear and home 0 0 0 0 0 0 0 1
	Delaysw(50);
	//LCD_COMM(0x0c);	//display on, no cursor, no blink
	//DelayTim0(t_5ms);
	LCD_COMM(0x0c);		// dp. on, curs. on, blink on
	Delaysw(50);
	LCD_COMM(0x06);		// Entry mode 0 0 0 0 0 1 I/D S dp. on, curs. on, blink on
	Delaysw(50);
}

//Set LCD line (1 or 2) to write
void SetLine(char line)
{
	if(line == 1)
	{
		LCD_COMM(LINE1);
	}
	else
	{
		LCD_COMM(LINE2);
	}
}

void CLR_DISPLAY(void)
{
	LCD_COMM(CLR_DISP);
}

//copy l pieces of characters
void cpy_str(char * sd, char * ss, int l)  // sd[l-1:0] = ss[l-1:0]
{	
	int i;
	int len;
	int sl;
	char tmp;
	
	sl = strlen(ss);
	
	if(l > sl)
	{
		len = sl;
	}
	else
	{
		len = l;
	}
	
	for(i=0; i<len; i++)
	{
		sd[i] = ss[i];
	}
	sd[i]=0;
}

//complete string shorter then 16 char with space
void complete_str16(char * s)	
{
	int l = strlen(s);
	if(l < 16)
	{
		for(int i=0; i<=(16-l); i++ )
		{
			strcat(s," ");
		}
	}	
}

//write string to LCD line
void LCD_write_str(char * s, char line)
{
	//char tmpstr[20];
	
	cpy_str(tmpstr1,s,16);
	complete_str16(tmpstr1);
	
	do{} while(!LCD_WR_RDY);
		
	strcpy(ITstr,tmpstr1);
	//StrPtr = ITstr;
	StrIndex = -1;
	LCDline = line;
	LCD_WR_RDY = 0;
}


//write string and integer to LCD line
void LCD_write_str_int(char * s, int data, char side, char line)
{
	//char tmpstr[20];
	//char tempdat[20];
	int slen;
	cli();	
	cpy_str(tmpstr1,s,14);
	itoa(data,tmpstr2,10);
	if(side == right)
	{
		strcat(tmpstr1,tmpstr2);	
	}
	else
	{
		strcat(tmpstr2,tmpstr1);
		strcpy(tmpstr1,tmpstr2);
	}
	
	complete_str16(tmpstr1);
	sei();
	do{} while(!LCD_WR_RDY);
		
	strcpy(ITstr,tmpstr1);

	StrIndex = -1;
	LCDline = line;
	LCD_WR_RDY = 0;
}

//write string to LCD line1 and line2 
void LCD_write_longstr(char * s)
{	
	//char tmpstr[16];
	char *strptr = s+16;
	int len = strlen(s);
	LCD_write_str(s,line1);
	if(len>16)
	{
		strncpy(tmpstr1,strptr,len-16);
		tmpstr1[len-16] = 0;
		complete_str16(tmpstr1);
		LCD_write_str(tmpstr1,line2);
	}
}

//set back light (value: 0-15)
void LCD_light(uint8_t bright)
{
	if(bright == 0)
	{
		TCCR0A &= ~(1 << COM0B1);
	}
	else
	{
		TCCR0A |= (1 << COM0B1);
		OCR0B = bright & 0x0f;
	}
}