/*
 * ServoPWM.c
 *
 * Created: 3/13/2015 3:13:51 PM
 *  Author: Zoltan Benesoczky
 */ 

#include <avr/io.h>
#include <avr/interrupt.h>
#include <inttypes.h>
#include <stdlib.h>
#include <string.h>
#include "BooleConst.h"
#include "LCD_hlr.h"
#include "Keyboard_def.h"
#include "Keyboard_hlr.h"

// software or hardware pwm
//#define swpwm

//use TIMER2
//        D7     D6     D5     D4     D3    D2   D1    D0
//TCCR2A: COM2A1 COM2A0 COM2B1 COM2B0 -     -    WGM21 WGM20
//        0      0      1      0      0     0    0     1
//TCCR2B: FOC2A  FOC2B  -      -      WGM22 CS22 CS21  CS20
//        0      0      0      0      0     1    1     0
//Phase correct PWM mode, div=256, (time resolution 1000*256/8e6 = 0.032ms, cycle time: 2*0.032*256 = 16.384ms)
//TH = 1msec when OCR2B = 16
#define mode2a 0x21
#define mode2b 0x06

void Tim2Init(void)
{
	TCCR2A = mode2a;
	TCCR2B = mode2b;
}

//set position: 0-31
void SetServoPosition(unsigned char pos)
{
	pos = pos & 0x1f;
	OCR2B = 16 + pos;	//Minimum position: TH=1ms,  maximum position: TH=3msec cycle time = 16msec
}

void SW_PWM(void);

static volatile int timcnt2;
static volatile unsigned char ovr2;

void Timer0ItExtension(void)
{
	KeyScan();
#ifdef swpwm	
	SW_PWM();
#endif
}


#define ServoPort PORTD
#define ServoPortDir DDRD
#define ServoTrig 3
#define InitServo ServoPortDir|= (1<<ServoTrig)

static volatile int servo_pos;

#ifdef swpwm
#define SetServoTrig ServoPort |= (1<<ServoTrig)
#define ClrServoTrig ServoPort &= ~(1<<ServoTrig)
//16msec
#define pwm_cnt_max (int)40
//1msec
#define pwm_min (int)2
static volatile int pwm_cnt;

void SW_PWM(void)
{
	if(pwm_cnt < pwm_cnt_max)
	{
		pwm_cnt++;
	}
	else
	{
		pwm_cnt = 0;
		SetServoTrig;
	}
	
	if(pwm_cnt == pwm_min + servo_pos)
	{
		ClrServoTrig;
	}
}
#endif

void InitCard(void)
{
	LCD_init();
	InitServo;
	KeyInit();
	#ifndef swpwm
	Tim2Init();		//set timer2 to phase syncron PWM mode
	#endif
	sei();
}


#ifdef swpwm
#define posmax 4
#else
#define posmax 31
#endif

main(void)
{
	#define servo_speed 200
	volatile uint8_t tmp, dir, brightness, servo_on;
	char LCDstr[20];
	KeyType key;
	
	InitCard();
	servo_pos = 0;
	servo_on = FALSE;
	brightness = 0;
	LCD_light(0);
	LCD_write_str("   Servo demo   ", line1);
	LCD_write_str_int("position: ", servo_pos, right, line2);
	dir = 1;
	SetServoPosition(servo_pos);
	
	while(1)
	{
		KeyPressed(&key);
		if(key.pressed)
		{				
			if(key.value == (1<<BUTTON))
			{
				servo_on = ~servo_on;
				StartTime0(servo_speed);
			}
		}
		
		if(servo_on)
		{
			if(Time0Done())
			{				
				if(dir == 1)
				{
					if(servo_pos < posmax)
					{
						servo_pos = (servo_pos + 1);
						#ifndef swpwm
						SetServoPosition(servo_pos);
						#endif
					}
					else
					{
						servo_pos = (servo_pos - 1);
						#ifndef swpwm
						SetServoPosition(servo_pos);
						#endif
						dir = 0;
					}
				}
				else
				{
					if(servo_pos > 0)
					{
						servo_pos = (servo_pos - 1);
						#ifndef swpwm
						SetServoPosition(servo_pos);
						#endif
					}
					else
					{
						servo_pos = (servo_pos + 1);						
						#ifndef swpwm
						SetServoPosition(servo_pos);
						#endif
						dir = 1;
					}
				}
				LCD_write_str_int("position: ", servo_pos, right, line2);
				StartTime0(servo_speed);
			}			
		}
	}
}