#include <avr/io.h>
#include <avr/interrupt.h>
#include "ADC_hlr.h"

//------------------ ADC ----------------------------
//          D7     D6     D5     D4     D3     D2     D1   D0
//ADMUX: REFS1   REFS0  ADLAR     -    MUX3   MUX2   MUX1 MUX0
//           0      1      0      0      0     0      0     0 
//			 0      0 external:AREF      MUX3-0: 0-7 ADC0-ADC7
//			 0      1 AVCC				 MUX3-0: 8   temperature sensor
//			 1		0 -					 MUX3-0: 0e  1.1V
//			 1		1 internal: 1.1V	 MUX3-0: 0f	 0V (GND)
//Vref: AVcc, Input: ADC0, No Gain, right justified


//          D7     D6     D5     D4     D3     D2     D1    D0
//ADCSRA: ADEN   ADSC   ADATE   ADIF   ADIE   ADPS2  ADPS1 ADPS0
//          1      1      1      0      1     1      1      1 
//                                            0      0      0	2  clk division
//                                            0      0      1	2
//                                            0      1      0	4
//                                            0      1      1	8
//                                            1      0      0	16
//                                            1      0      1	32
//                                            1      1      0	64
//                                            1      1      1	128

//ADEN: , Input: ADC0, AD start, free run, IT enable, div:128





// 16e6/128/13=9615,4Hz (sample frequ) 
#define ADMUX0 0x40		//ref AVCC
#define ADCSRA0 0xEF
#define AD_bufflng 32
static volatile unsigned int AD_buff[AD_bufflng];
static volatile unsigned int *AD_buffp;
static volatile int AD_wp = 0;
static volatile int AD_rp = 0;
static volatile int AD_num = 0;
static volatile unsigned int AD_dat = 0;
static volatile unsigned char AD_Ready;
static volatile int adcnt;
static volatile int sw_div0;

void AD_ITDIS(void)
{
  ADCSRA = ADCSRA & 0xf7;
}

void AD_ITEN(void)
{
  ADCSRA = ADCSRA | 0xe8; 
}

void AD_FrequSet(unsigned char fdiv)
{
	unsigned char tmp;
	AD_ITDIS();
    if(fdiv < 8)  
    {
      tmp = ADCSRA;
      tmp = tmp & ~0x07;
      tmp = tmp | fdiv;
    } 
	AD_ITEN();   
}

void AD_ChannelSet(unsigned char ch)
{
 unsigned char tmp;
	ch &= 0x0f;
    tmp = ADMUX;
    tmp = tmp & ~0x0f;
    tmp = tmp | ch;
    ADMUX = tmp;        
}

void ADC_BuffReset(void)
{
	AD_buffp = &AD_buff[0];
	AD_ITDIS();
	AD_wp = 0;
	AD_rp = 0;
	AD_num = 0;
	AD_dat = 0;
	AD_ITEN();
}

void ADC_Init(unsigned char Ref, unsigned char ch, unsigned char trigg, unsigned char fdiv, unsigned int sw_div)
{	
	ADC_BuffReset();
	ADMUX = Ref;	
	ADCSRA = ADCSRA0;
    ADCSRB = trigg;
	sw_div0 = sw_div-1;	// division in the interrupt
	adcnt = sw_div0;
    AD_ChannelSet(ch);
	AD_FrequSet(fdiv);
}



//get the result from ADC

ISR(ADC_vect)
{
//#define ADCtest
         
    AD_dat = ADCL;   // read AD data, ADCL must read first!
    AD_dat = AD_dat | ((unsigned int)ADCH << 8);
      
    if(adcnt > 0)
    {
		adcnt--; 
    }
    else
    {
      adcnt = sw_div0;
#ifdef ADCtest  
      //ADbuf[wp] = AD_dat; // testing
	  *AD_buffp = AD_dat;
      AD_dat++;           // testing
#else      
      //ADbuf[wp] =AD_dat; 
	  *(AD_buffp+AD_wp) = AD_dat;
#endif    
      AD_wp =(AD_wp+1) % AD_bufflng;	// pointer modulo lltsa
      if (AD_num < AD_bufflng)	// ha nincs fellrs
      {
          AD_num++;
      }
      else					// ha fellrtuk a legrgebbit
      {  
		//buf_ovr = 1;	// jelzs, tele a buffer, elvesz a legrgebbi adat!
        //LED7_TOGLE;
		AD_num = AD_bufflng;
		AD_rp = AD_wp;		// olvas pointer az r utn mutat 							// mivel mostmr ez a legrgebbi adat
      }
    }
}

// bufferbol olvaso rutin
unsigned char Get_ADC(unsigned int *ADvar)
{
  AD_ITDIS();
    if (AD_num > 0)
    {
	    *ADvar = *(AD_buffp + AD_rp);
	    AD_rp = (AD_rp + 1) % AD_bufflng;
	    AD_num--;
		AD_ITEN();
	    return(1);
    }
    else
    {
		AD_ITEN();
	    return(0);
    }
}


unsigned char Get_ADC_Voltage(double *ADvoltage,double ADref)
{
	unsigned int int_value;
	if(Get_ADC(&int_value))
	{
		*ADvoltage = (double)int_value*ADref/1024;
		return(1);
	}
	else
	{
		return(0);
	}
}
